/**
 * @file  file_device.h
 * @brief Schnittstelle fuer die Klasse FileDevice.
 */
//////////////////////////////////////////////////////////////////////
#pragma once
#ifndef FILE_DEVICE_H
#define FILE_DEVICE_H

#include "common/device_info.h"

#include "file/file_intel_hex.h"
#include "file/file_display_fonts.h"
#include "file/file_display_grafics.h"
#include "file/file_device_text.h"

#include "interface/exchange_classes.h"
#include "interface/device_update_classes.h"
#include "interface/device_structs.h"

#include "datafox/communication/module_hardware.h"
#include "common/software_version.h"

class FileDevice : public FileIntelHex
{
public:
    FileDevice();
    virtual ~FileDevice();

public:
	void resetData();

    unsigned int GetHeaderVersionFile();
    unsigned int GetHeaderVersionTimestamp( std::string &timestamp );
    unsigned int GetHeaderVersionFirmware( std::string &versionFirmware );
    SoftwarebuildVersion GetHeaderVersionFirmware();
    StructSoftwareInfo GetHeaderSoftwareInfo();

    SDFFileDeviceHeader GetHeader( const std::string &filePath );
    int CheckVersionFile( SDFFileDeviceHeader &header );
    unsigned int GetVersionFile( const std::string &filePath );
    unsigned int GetVersionFirmware( const std::string &filePath, std::string &versionFirmware ); // max. 32 Zeichen Firmwareversion

    bool load( const std::string &filePath );
    bool save( const std::string &filePath, SDFFileDeviceHeader &header );
    int build( const std::string &filePath, SDFFileDeviceHeader &header, const std::string &fileFirmware, const std::string &fileDisplayFonts, const std::string &fileDisplayGrafics, const std::string &fileDeviceText );

    bool loadBin( const std::string &filePath );
    bool saveBin( const std::string &filePath, SDFFileDeviceHeader &header );

    StructSoftwareInfo SoftwareInfoStringToStruct( const std::string &str );
    bool SoftwareInfoStructToString( std::string &str, StructSoftwareInfo &swInfo );

    StructSoftwareInfo CheckSoftwareInfo( const StructSoftwareInfo &swInfo );

    int CompareInfos( const StructHardwareInfo &hwInfo, const StructSoftwareInfo &swInfo, bool checkBordRevision = true );
    int CompareInfos( const StructHardwareInfo &hwInfo, const StructSoftwareInfo &swInfo, const std::string &versionFirmware, bool checkBordRevision = true );
    int SoftwareInfoToString( const StructSoftwareInfo &swInfo, std::string &info );

	std::vector<unsigned char> firmwareContent()
	{
		return m_fileRawFirmware.content();
	}
	unsigned int firmwareFirstAddress()
	{
		return m_fileRawFirmware.firstAddress();
	}
	unsigned int firmwareLastAddress()
	{
		return m_fileRawFirmware.lastAddress();
	}
    FileIntelHex *GetFileRawFirmware();
    FileDisplayFonts *GetFileDisplayFonts();
    FileDisplayGrafics *GetFileDisplayGrafics();
    FileDeviceText *GetFileDeviceText();

    std::string m_fileName;
protected:
    // Der Dateiheader wird ueber einen Puffer verwaltet, somit ist eine
    // vergroesserte Struktur auch von aelteren Softwarestaenden lesbar und
    // eine Pruefung des crcHead durch info:sizeData moeglich.
    unsigned char m_headBuf[IV_DFFILEDEVICE_SIZE_HEADER];
    SDFFileDeviceHeader *m_header;
    FileIntelHex m_fileRawFirmware;
    FileDisplayFonts m_fileDisplayFonts;
    FileDisplayGrafics m_fileDisplayGrafics;
    FileDeviceText m_fileDeviceText;

public:
    Datafox::FileHeader &fileHeader()
    {
        return _fileHeader;
    }

    Datafox::DeviceUpdateNode &rootNode()
    {
        return _deviceUpdateNode;
    }
private:
    // Dateiheader
    Datafox::FileHeader _fileHeader;
    // Verwaltungsstrukturen fuer DeviceUpdateSegmente
    Datafox::DeviceUpdateNode _deviceUpdateNode;
    // Vector fuer die Binaerdatendatei
    std::vector<unsigned char> bufferBin_;
    bool readSerialize();
    bool writeSerialize( const SDFFileDeviceHeader &header );

protected:
    DeviceInfo _deviceInfo;
};

#endif

