#ifndef DEVICE_UPDATE_CLASSES_H
#define DEVICE_UPDATE_CLASSES_H

#include "device_update_structs.h"
#include "exchange_classes.h"

namespace Datafox
{
	
class DeviceUpdateSegmentFirmware
{
public:
    typedef std::array<unsigned char, sizeof(StructDeviceUpdateSegmentFirmware)> DataType;
    typedef std::vector<unsigned char> SerializeType;

public:
    DeviceUpdateSegmentFirmware();
    DeviceUpdateSegmentFirmware( const DeviceUpdateSegmentFirmware &segment );

public:
    EnumStructId id() const;
    int size() const;
    StructDeviceUpdateSegmentFirmware &data();

    SerializeType::size_type sizeSegmentData() const;
    SerializeType &segmentData();

public:
    void resetData();
    void writeSerialize( SerializeType &buffer );
    void readSerialize( SerializeType::const_iterator iter );

protected:
    void writeSerialize( SerializeType::iterator iter );

protected:
    EnumStructId _id;
    DataType _structData;
    SerializeType segmentData_;

#ifdef _DEBUG
    StructDeviceUpdateSegmentFirmware *_data;
#endif
};

class DeviceUpdateSegment
{
public:
    typedef std::array<unsigned char, sizeof(StructDeviceUpdateSegment)> DataType;
    typedef std::vector<unsigned char> SerializeType;

public:
    DeviceUpdateSegment();
    DeviceUpdateSegment( const DeviceUpdateSegment &segment );

public:
    EnumStructId id() const;
    int size() const;
    StructDeviceUpdateSegment &data();

    SerializeType::size_type sizeSegmentData() const;
    SerializeType &segmentData();

public:
    void resetData();
    void writeSerialize( SerializeType &buffer );
    void readSerialize( SerializeType::const_iterator iter );

protected:
    void writeSerialize( SerializeType::iterator iter );

protected:
    EnumStructId _id;
    DataType _structData;
    SerializeType segmentData_;

#ifdef _DEBUG
    StructDeviceUpdateSegment *_data;
#endif
};


class DeviceUpdateNode
{
public:
    typedef std::array<unsigned char, sizeof(StructDeviceUpdateNode)> DataType;
    typedef std::vector<unsigned char> SerializeType;
    typedef std::vector<DeviceUpdateSegment> typeSegments;

public:
    DeviceUpdateNode();

public:
    void resetData();
    bool writeSerialize( SerializeType &buffer );
    bool readSerialize( const SerializeType &buffer );

protected:
    void writeSerialize( SerializeType::iterator iter );
    void readSerialize( SerializeType::const_iterator iter );

public:
    EnumStructId id() const;
    std::size_t size() const;
    StructDeviceUpdateNode &data();

protected:
    EnumStructId _id;
    DataType _structData;
#ifdef _DEBUG
    StructDeviceUpdateNode *_data;
#endif

public:
    void setSegmentFirmware( const StructDeviceUpdateSegmentFirmware &dusf, const SerializeType &segmentData );
    const StructDeviceUpdateSegmentFirmware &getSegmentFirmware();
    const SerializeType &getSegmentDataFirmware();

    std::size_t countSegments();
    void appendSegment( const StructDeviceUpdateSegment &segment, const SerializeType &segmentData );
    const DeviceUpdateSegment &getSegment( int index );
    const SerializeType &getSegmentData( int index );

protected:
    DeviceUpdateSegmentFirmware _segmentFirmware;
    StructArray _array;
    typeSegments _segments;
};

}

#endif
